@@ -2,6 +2,7 @@
 
 /* 
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2002 Red Hat, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +23,7 @@
 #ifndef META_WINDOW_H
 #define META_WINDOW_H
 
+#include <config.h>
 #include "screen.h"
 #include "util.h"
 #include "stack.h"
@@ -31,8 +33,8 @@
 
 typedef struct _MetaGroup MetaGroup;
 
-typedef void (*MetaWindowForeachFunc) (MetaWindow *window,
-                                       void       *data);
+typedef gboolean (*MetaWindowForeachFunc) (MetaWindow *window,
+                                           void       *data);
 
 typedef enum
 {
@@ -79,6 +81,7 @@ struct _MetaWindow
   char *role;
   char *sm_client_id;
   char *wm_client_machine;
+  char *startup_id;
 
   int net_wm_pid;
   
@@ -211,6 +214,18 @@ struct _MetaWindow
 
   /* Transient parent is a root window */
   guint transient_parent_is_root_window : 1;
+
+  /* Info on which props we got our attributes from */
+  guint using_net_wm_name : 1; /* vs. plain wm_name */
+  guint using_net_wm_icon_name : 1; /* vs. plain wm_icon_name */
+
+  /* has a shape mask */
+  guint has_shape : 1;
+  
+#ifdef HAVE_XSYNC
+  /* XSync update counter */
+  XSyncCounter update_counter;
+#endif
   
   /* Number of UnmapNotify that are caused by us, if
    * we get UnmapNotify with none pending then the client
@@ -317,6 +332,10 @@ void        meta_window_resize_with_gravity (MetaWindow  *window,
                                              int          h,
                                              int          gravity);
 
+
+void        meta_window_fill_horizontal     (MetaWindow  *window);
+void        meta_window_fill_vertical       (MetaWindow  *window);
+
 /* This recalcs the window/frame size, and recalcs the frame
  * size/contents as well.
  */
@@ -372,7 +391,7 @@ gboolean meta_window_client_message    (MetaWindow *window,
 gboolean meta_window_notify_focus      (MetaWindow *window,
                                         XEvent     *event);
 
-int      meta_window_set_current_workspace_hint (MetaWindow *window);
+void     meta_window_set_current_workspace_hint (MetaWindow *window);
 
 unsigned long meta_window_get_net_wm_desktop (MetaWindow *window);
 
@@ -402,21 +421,25 @@ void meta_window_get_work_area (MetaWindow    *window,
 gboolean meta_window_same_application (MetaWindow *window,
                                        MetaWindow *other_window);
 
-#define META_WINDOW_IN_NORMAL_TAB_CHAIN(w) \
+#define META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE(w) \
   ((w)->type != META_WINDOW_DOCK && (w)->type != META_WINDOW_DESKTOP)
-
+#define META_WINDOW_IN_NORMAL_TAB_CHAIN(w) \
+  (((w)->input || (w)->take_focus) && META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE (w))
 #define META_WINDOW_IN_DOCK_TAB_CHAIN(w) \
-  (! META_WINDOW_IN_NORMAL_TAB_CHAIN (w))
+  (((w)->input || (w)->take_focus) && ! META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE (w))
 
 void meta_window_refresh_resize_popup (MetaWindow *window);
 
 void meta_window_free_delete_dialog (MetaWindow *window);
 
-void meta_window_foreach_transient (MetaWindow            *window,
-                                    MetaWindowForeachFunc  func,
-                                    void                  *data);
-gboolean meta_window_is_ancestor_of_transient (MetaWindow *window,
-                                               MetaWindow *transient);
+void     meta_window_foreach_transient        (MetaWindow            *window,
+                                               MetaWindowForeachFunc  func,
+                                               void                  *data);
+gboolean meta_window_is_ancestor_of_transient (MetaWindow            *window,
+                                               MetaWindow            *transient);
+void     meta_window_foreach_ancestor         (MetaWindow            *window,
+                                               MetaWindowForeachFunc  func,
+                                               void                  *data);
 
 gboolean meta_window_warp_pointer (MetaWindow *window,
                                    MetaGrabOp  grab_op);
@@ -430,4 +453,9 @@ void meta_window_update_resize_grab_op (MetaWindow *window,
 
 void meta_window_update_layer (MetaWindow *window);
 
+gboolean meta_window_get_icon_geometry (MetaWindow    *window,
+                                        MetaRectangle *rect);
+
+const char* meta_window_get_startup_id (MetaWindow *window);
+
 #endif
